@@ -28,6 +28,7 @@
 #define _GNU_SOURCE
 
 #include <math.h>
+#include <string.h>
 
 #ifndef LEGACY
 #    include <hildon/hildon-help.h>
@@ -46,6 +47,9 @@
 
 #include "display.h"
 #include "gps.h"
+#include "aprs.h"
+#include "aprs_kiss.h"
+#include "aprs_display.h"
 #include "gdk-pixbuf-rotate.h"
 #include "gpx.h"
 #include "maps.h"
@@ -55,53 +59,6 @@
 #include "settings.h"
 #include "util.h"
 
-static GtkWidget *_menu_route_open_item = NULL;
-static GtkWidget *_menu_route_download_item = NULL;
-static GtkWidget *_menu_route_save_item = NULL;
-static GtkWidget *_menu_route_distnext_item = NULL;
-static GtkWidget *_menu_route_distlast_item = NULL;
-static GtkWidget *_menu_route_reset_item = NULL;
-static GtkWidget *_menu_route_clear_item = NULL;
-static GtkWidget *_menu_track_open_item = NULL;
-static GtkWidget *_menu_track_save_item = NULL;
-static GtkWidget *_menu_track_insert_break_item = NULL;
-static GtkWidget *_menu_track_insert_mark_item = NULL;
-static GtkWidget *_menu_track_distlast_item = NULL;
-static GtkWidget *_menu_track_distfirst_item = NULL;
-static GtkWidget *_menu_track_clear_item = NULL;
-static GtkWidget *_menu_poi_import_item = NULL;
-static GtkWidget *_menu_poi_download_item = NULL;
-static GtkWidget *_menu_poi_browse_item = NULL;
-static GtkWidget *_menu_poi_categories_item = NULL;
-static GtkWidget *_menu_maps_submenu = NULL;
-static GtkWidget *_menu_maps_mapman_item = NULL;
-static GtkWidget *_menu_maps_repoman_item = NULL;
-static GtkWidget *_menu_view_zoom_in_item = NULL;
-static GtkWidget *_menu_view_zoom_out_item = NULL;
-static GtkWidget *_menu_view_rotate_clock_item = NULL;
-static GtkWidget *_menu_view_rotate_counter_item = NULL;
-static GtkWidget *_menu_view_rotate_reset_item = NULL;
-static GtkWidget *_menu_view_pan_up_item = NULL;
-static GtkWidget *_menu_view_pan_down_item = NULL;
-static GtkWidget *_menu_view_pan_left_item = NULL;
-static GtkWidget *_menu_view_pan_right_item = NULL;
-static GtkWidget *_menu_view_pan_north_item = NULL;
-static GtkWidget *_menu_view_pan_south_item = NULL;
-static GtkWidget *_menu_view_pan_west_item = NULL;
-static GtkWidget *_menu_view_pan_east_item = NULL;
-static GtkWidget *_menu_view_show_zoomlevel_item = NULL;
-static GtkWidget *_menu_view_show_comprose_item = NULL;
-static GtkWidget *_menu_view_show_velvec_item = NULL;
-static GtkWidget *_menu_view_goto_latlon_item = NULL;
-static GtkWidget *_menu_view_goto_address_item = NULL;
-static GtkWidget *_menu_view_goto_gps_item = NULL;
-static GtkWidget *_menu_view_goto_nextway_item = NULL;
-static GtkWidget *_menu_view_goto_nearpoi_item = NULL;
-static GtkWidget *_menu_settings_item = NULL;
-static GtkWidget *_menu_help_item = NULL;
-static GtkWidget *_menu_about_item = NULL;
-static GtkWidget *_menu_close_item = NULL;
-
 /****************************************************************************
  * BELOW: ROUTE MENU ********************************************************
  ****************************************************************************/
@@ -341,8 +298,10 @@ menu_cb_track_insert_mark(GtkMenuItem *item)
     }
 
     unit2latlon(_pos.unitx, _pos.unity, lat, lon);
-    lat_format(lat, tmp1);
-    lon_format(lon, tmp2);
+    
+    format_lat_lon(lat, lon, tmp1, tmp2);
+    //lat_format(lat, tmp1);
+    //lon_format(lon, tmp2);
     p_latlon = g_strdup_printf("%s, %s", tmp1, tmp2);
     gtk_label_set_text(GTK_LABEL(lbl_latlon), p_latlon);
     g_free(p_latlon);
@@ -515,6 +474,15 @@ menu_cb_maps_repoman(GtkMenuItem *item)
     return TRUE;
 }
 
+static gboolean
+menu_cb_maps_repodown(GtkMenuItem *item)
+{
+    printf("%s()\n", __PRETTY_FUNCTION__);
+    repoman_download();
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
 static gboolean
 menu_cb_maps_select(GtkMenuItem *item, gpointer new_repo)
 {
@@ -563,6 +531,35 @@ menu_cb_maps_auto_download(GtkMenuItem *item)
  * ABOVE: MAPS MENU *********************************************************
  ****************************************************************************/
 
+/****************************************************************************
+ * BELOW: LAYERS MENU *******************************************************
+ ****************************************************************************/
+
+static gboolean
+menu_cb_layers_toggle(GtkCheckMenuItem *item, gpointer layer)
+{
+    RepoData* rd = (RepoData*)layer;
+
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    rd->layer_enabled = !rd->layer_enabled;
+
+    /* refresh if layer is on top of active map */
+    if (repo_is_layer (_curr_repo, rd)) {
+        /* reset layer's countdown */
+        rd->layer_refresh_countdown = rd->layer_refresh_interval;
+        map_cache_clean ();
+        map_refresh_mark (TRUE);
+    }
+
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
+/****************************************************************************
+ * ABOVE: LAYERS MENU *******************************************************
+ ****************************************************************************/
+
 /****************************************************************************
  * BELOW: VIEW/ZOOM MENU ****************************************************
  ****************************************************************************/
@@ -826,6 +823,7 @@ menu_cb_view_goto_latlon(GtkMenuItem *item)
                 1, 2, 0, 1, GTK_FILL, 0, 2, 4);
         gtk_entry_set_width_chars(GTK_ENTRY(txt_lat), 16);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
+#ifdef MAEMO_CHANGES
 #ifndef LEGACY
         g_object_set(G_OBJECT(txt_lat), "hildon-input-mode",
                 HILDON_GTK_INPUT_MODE_FULL, NULL);
@@ -833,6 +831,7 @@ menu_cb_view_goto_latlon(GtkMenuItem *item)
         g_object_set(G_OBJECT(txt_lat), HILDON_AUTOCAP, FALSE, NULL);
         g_object_set(G_OBJECT(txt_lat), HILDON_INPUT_MODE_HINT,
                 HILDON_INPUT_MODE_HINT_ALPHANUMERICSPECIAL, NULL);
+#endif
 #endif
 
         gtk_table_attach(GTK_TABLE(table),
@@ -845,6 +844,7 @@ menu_cb_view_goto_latlon(GtkMenuItem *item)
                 1, 2, 1, 2, GTK_FILL, 0, 2, 4);
         gtk_entry_set_width_chars(GTK_ENTRY(txt_lon), 16);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
+#ifdef MAEMO_CHANGES
 #ifndef LEGACY
         g_object_set(G_OBJECT(txt_lon), "hildon-input-mode",
                 HILDON_GTK_INPUT_MODE_FULL, NULL);
@@ -852,18 +852,21 @@ menu_cb_view_goto_latlon(GtkMenuItem *item)
         g_object_set(G_OBJECT(txt_lon), HILDON_AUTOCAP, FALSE, NULL);
         g_object_set(G_OBJECT(txt_lon), HILDON_INPUT_MODE_HINT,
                 HILDON_INPUT_MODE_HINT_ALPHANUMERICSPECIAL, NULL);
+#endif
 #endif
     }
 
     /* Initialize with the current center position. */
     {
-        gchar buffer[32];
+        gchar buffer1[32];
+        gchar buffer2[32];
         gdouble lat, lon;
         unit2latlon(_center.unitx, _center.unity, lat, lon);
-        lat_format(lat, buffer);
-        gtk_entry_set_text(GTK_ENTRY(txt_lat), buffer);
-        lon_format(lon, buffer);
-        gtk_entry_set_text(GTK_ENTRY(txt_lon), buffer);
+        //lat_format(lat, buffer1);
+        //lon_format(lon, buffer2);
+        format_lat_lon(lat, lon, buffer1, buffer2);
+        gtk_entry_set_text(GTK_ENTRY(txt_lat), buffer1);
+        gtk_entry_set_text(GTK_ENTRY(txt_lon), buffer2);
     }
 
     gtk_widget_show_all(dialog);
@@ -1246,6 +1249,125 @@ menu_cb_view_fullscreen(GtkMenuItem *item)
     return TRUE;
 }
 
+/****************************************************************************
+ * BELOW: APRS MENU *********************************************************
+ ****************************************************************************/
+#ifdef INCLUDE_APRS
+
+static gboolean menu_cb_aprs_settings(GtkMenuItem *item)
+{
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    gboolean aprs_inet_config_changed = FALSE;
+    gboolean aprs_tty_config_changed = FALSE;
+
+    aprs_settings_dialog(&aprs_inet_config_changed, &aprs_tty_config_changed);
+    
+    if(aprs_inet_config_changed)
+    {
+        gtk_widget_set_sensitive(GTK_WIDGET(_menu_enable_aprs_inet_item), _aprs_enable);
+        gtk_widget_set_sensitive(GTK_WIDGET(_menu_enable_aprs_tty_item), _aprs_enable);
+        gtk_widget_set_sensitive(GTK_WIDGET(_menu_list_aprs_stations_item), _aprs_enable);
+        gtk_widget_set_sensitive(GTK_WIDGET(_menu_list_aprs_messages_item), _aprs_enable);
+
+    	
+    	aprs_server_disconnect();
+    	
+    	if(_aprs_enable && _aprs_inet_enable)
+    	{
+    		aprs_server_connect();
+    	}
+    }
+    
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
+static gboolean menu_cb_enable_tty_aprs(GtkMenuItem *item)
+{
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    if(_aprs_enable)
+    {
+
+	    if((_aprs_tty_enable = gtk_check_menu_item_get_active(
+	                GTK_CHECK_MENU_ITEM(_menu_enable_aprs_tty_item))))
+	        aprs_tty_connect();
+	    else
+	        aprs_tty_disconnect();
+    }
+    
+
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
+
+static gboolean
+menu_cb_enable_inet_aprs(GtkMenuItem *item)
+{
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    if(_aprs_enable)
+    {
+
+	    if((_aprs_inet_enable = gtk_check_menu_item_get_active(
+	                GTK_CHECK_MENU_ITEM(_menu_enable_aprs_inet_item))))
+	        aprs_server_connect();
+	    else
+	        aprs_server_disconnect();
+
+
+    }
+    else
+    {
+    	// APRS not enabled
+    }
+    
+
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
+static gboolean menu_cb_list_aprs_messages(GtkMenuItem *item)
+{
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    if(_aprs_enable)
+    {
+    	list_messages();
+    }
+    else
+    {
+    	// APRS not enabled
+    }
+    
+
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;	
+}
+
+static gboolean
+menu_cb_list_aprs_stations(GtkMenuItem *item)
+{
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    if(_aprs_enable)
+    {
+    	list_stations();
+    }
+    else
+    {
+    	// APRS not enabled
+    }
+    
+
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
+#endif // INCLUDE_APRS
+
 /****************************************************************************
  * BELOW: GPS MENU **********************************************************
  ****************************************************************************/
@@ -1373,9 +1495,27 @@ menu_maps_remove_repos()
         gtk_widget_destroy(gtk_container_get_children(
                     GTK_CONTAINER(_menu_maps_submenu))->data);
     }
+
+    menu_layers_remove_repos ();
+
     vprintf("%s(): return\n", __PRETTY_FUNCTION__);
 }
 
+
+void
+menu_layers_remove_repos()
+{
+    GList *child;
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    /* Delete one menu item for each repo. */
+    while ((child = gtk_container_get_children(GTK_CONTAINER(_menu_layers_submenu))))
+        gtk_widget_destroy (child->data);
+
+    vprintf("%s(): return\n", __PRETTY_FUNCTION__);
+}
+
+
 void
 menu_maps_add_repos()
 {
@@ -1414,9 +1554,65 @@ menu_maps_add_repos()
     }
 
     gtk_widget_show_all(_menu_maps_submenu);
+    menu_layers_add_repos ();
+
     vprintf("%s(): return\n", __PRETTY_FUNCTION__);
 }
 
+
+void
+menu_layers_add_repos()
+{
+    GList *curr;
+
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    for(curr = _repo_list; curr; curr = curr->next)
+    {
+        RepoData* rd = (RepoData*)curr->data;
+        GtkWidget *item, *submenu = NULL, *layer_item;
+
+        /* if repository doesn't have layers, skip it */
+        if (!rd->layers)
+            continue;
+
+        /* if it has only one layer, add just one check menu item */
+        if (!rd->layers->layers) {
+            gchar *title = g_malloc (strlen (rd->name) + strlen (rd->layers->name) + 3);
+
+            sprintf (title, "%s[%s]", rd->name, rd->layers->name);
+
+            rd = rd->layers;
+            gtk_menu_append (_menu_layers_submenu, layer_item = gtk_check_menu_item_new_with_label (title));
+            gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (layer_item), rd->layer_enabled);
+            g_signal_connect (G_OBJECT (layer_item), "toggled", G_CALLBACK (menu_cb_layers_toggle), rd);
+            rd->menu_item = layer_item;
+        }
+        else {
+            /* append main repository menu item  */
+            gtk_menu_append (_menu_layers_submenu, item = gtk_menu_item_new_with_label(rd->name));
+
+            rd = rd->layers;
+            while (rd) {
+                if (!submenu)
+                    submenu = gtk_menu_new ();
+                gtk_menu_append (submenu, layer_item = gtk_check_menu_item_new_with_label (rd->name));
+                gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (layer_item), rd->layer_enabled);
+                g_signal_connect (G_OBJECT (layer_item), "toggled", G_CALLBACK (menu_cb_layers_toggle), rd);
+                rd->menu_item = layer_item;
+                rd = rd->layers;
+            }
+
+            if (submenu)
+                gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+        }
+    }
+
+    gtk_widget_show_all(_menu_layers_submenu);
+    vprintf("%s(): return\n", __PRETTY_FUNCTION__);
+}
+
+
 /**
  * Create the menu items needed for the drop down menu.
  */
@@ -1493,6 +1689,8 @@ menu_init()
     gtk_menu_append(submenu, _menu_poi_categories_item
             = gtk_menu_item_new_with_label(_("Categories...")));
 
+    _menu_layers_submenu = gtk_menu_new();
+
     /* The "Maps" submenu. */
     gtk_menu_append(menu, menu_item
             = gtk_menu_item_new_with_label(_("Maps")));
@@ -1501,12 +1699,16 @@ menu_init()
     gtk_menu_append(_menu_maps_submenu, gtk_separator_menu_item_new());
     gtk_menu_append(_menu_maps_submenu, _menu_maps_mapman_item
             = gtk_menu_item_new_with_label(_("Manage Maps...")));
-    gtk_menu_append(_menu_maps_submenu, _menu_maps_repoman_item
-            = gtk_menu_item_new_with_label(_("Manage Repositories...")));
     gtk_menu_append(_menu_maps_submenu, _menu_maps_auto_download_item
             = gtk_check_menu_item_new_with_label(_("Auto-Download")));
     gtk_check_menu_item_set_active(
             GTK_CHECK_MENU_ITEM(_menu_maps_auto_download_item),_auto_download);
+    gtk_menu_append(_menu_maps_submenu, gtk_separator_menu_item_new());
+    gtk_menu_append(_menu_maps_submenu, _menu_maps_repoman_item
+            = gtk_menu_item_new_with_label(_("Manage Repositories...")));
+    gtk_menu_append(_menu_maps_submenu, _menu_maps_repodown_item
+            = gtk_menu_item_new_with_label(
+                _("Download Sample Repositories...")));
     menu_maps_add_repos();
 
     gtk_menu_append(menu, gtk_separator_menu_item_new());
@@ -1517,6 +1719,11 @@ menu_init()
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),
             submenu = gtk_menu_new());
 
+    /* The View/Layers submenu */
+    gtk_menu_append(submenu, menu_item
+            = gtk_menu_item_new_with_label(_("Layers")));
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), _menu_layers_submenu);
+
     /* The "View"/"Zoom" submenu. */
     gtk_menu_append(submenu, menu_item
             = gtk_menu_item_new_with_label(_("Zoom")));
@@ -1676,6 +1883,47 @@ menu_init()
 
     gtk_menu_append(menu, gtk_separator_menu_item_new());
 
+#ifdef INCLUDE_APRS
+    /* The "APRS" submenu. */
+    gtk_menu_append(menu, menu_item
+            = gtk_menu_item_new_with_label(_("APRS")));
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),
+            submenu = gtk_menu_new());
+    
+    gtk_menu_append(submenu, _menu_enable_aprs_inet_item
+            = gtk_check_menu_item_new_with_label(_("Connect to APRS Server")));
+    gtk_check_menu_item_set_active(
+            GTK_CHECK_MENU_ITEM(_menu_enable_aprs_inet_item), _aprs_inet_enable);
+    gtk_widget_set_sensitive(GTK_WIDGET(_menu_enable_aprs_inet_item), _aprs_enable);
+    
+    
+    gtk_menu_append(submenu, _menu_enable_aprs_tty_item
+                = gtk_check_menu_item_new_with_label(_("Connect to TNC")));
+        gtk_check_menu_item_set_active(
+                GTK_CHECK_MENU_ITEM(_menu_enable_aprs_tty_item), _aprs_tty_enable);
+        gtk_widget_set_sensitive(GTK_WIDGET(_menu_enable_aprs_tty_item), _aprs_enable);
+        
+    
+    gtk_menu_append(submenu, _menu_list_aprs_stations_item
+                = gtk_menu_item_new_with_label(_("List Stations...")));
+        
+    gtk_widget_set_sensitive(GTK_WIDGET(_menu_list_aprs_stations_item), _aprs_enable);
+
+
+    gtk_menu_append(submenu, _menu_list_aprs_messages_item
+                = gtk_menu_item_new_with_label(_("List Messages...")));
+        
+    gtk_widget_set_sensitive(GTK_WIDGET(_menu_list_aprs_messages_item), _aprs_enable);
+
+    
+    gtk_menu_append(submenu, _menu_aprs_settings_item
+                    = gtk_menu_item_new_with_label(_("APRS Settings...")));
+    
+    
+    gtk_menu_append(menu, gtk_separator_menu_item_new());
+
+#endif // INCLUDE_APRS
+
     /* The other menu items. */
     gtk_menu_append(menu, _menu_settings_item
         = gtk_menu_item_new_with_label(_("Settings...")));
@@ -1739,6 +1987,8 @@ menu_init()
     /* Connect the "Maps" signals. */
     g_signal_connect(G_OBJECT(_menu_maps_repoman_item), "activate",
                       G_CALLBACK(menu_cb_maps_repoman), NULL);
+    g_signal_connect(G_OBJECT(_menu_maps_repodown_item), "activate",
+                      G_CALLBACK(menu_cb_maps_repodown), NULL);
     g_signal_connect(G_OBJECT(_menu_maps_mapman_item), "activate",
                       G_CALLBACK(menu_cb_maps_mapman), NULL);
     g_signal_connect(G_OBJECT(_menu_maps_auto_download_item), "toggled",
@@ -1822,6 +2072,24 @@ menu_init()
     g_signal_connect(G_OBJECT(_menu_gps_reset_item), "activate",
                       G_CALLBACK(menu_cb_gps_reset), NULL);
 
+#ifdef INCLUDE_APRS
+    /* Connect the "APRS" signals. */
+    g_signal_connect(G_OBJECT(_menu_enable_aprs_inet_item), "toggled",
+                      G_CALLBACK(menu_cb_enable_inet_aprs), NULL);
+
+    g_signal_connect(G_OBJECT(_menu_enable_aprs_tty_item), "toggled",
+                      G_CALLBACK(menu_cb_enable_tty_aprs), NULL);
+
+    g_signal_connect(G_OBJECT(_menu_list_aprs_stations_item), "activate",
+                          G_CALLBACK(menu_cb_list_aprs_stations), NULL);
+
+    g_signal_connect(G_OBJECT(_menu_list_aprs_messages_item), "activate",
+                              G_CALLBACK(menu_cb_list_aprs_messages), NULL);
+    
+    g_signal_connect(G_OBJECT(_menu_aprs_settings_item), "activate",
+                                  G_CALLBACK(menu_cb_aprs_settings), NULL);
+#endif // INCLUDE_APRS        
+    
     /* Connect the other menu item signals. */
     g_signal_connect(G_OBJECT(_menu_settings_item), "activate",
                       G_CALLBACK(menu_cb_settings), NULL);
